/*******************************************************************************
 * Copyright 2017 Frank Mnster
 *
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/
package de.frankmuenster.jameica.qrcodehandler;

import java.util.ArrayList;
import java.util.List;
import java.util.ListIterator;

import de.willuhn.logging.Level;
import de.willuhn.logging.Logger;

/**
 * Registry fuer die QR-Code Handler Neue Handler muessen sich mittels register() in der Registry anmelden
 *
 * @author munstfr
 * @version 1.1
 */
public final class QrCodeHandlerRegsistry {

    /**
     * Liste der registrierten QR-Code-Handlers
     */
    private static List<IQrCodeHandler> qrCodeHandlers = new ArrayList<>();

    private QrCodeHandlerRegsistry() {
    }

    /**
     * Neue Qr-Code Handler koennen sich hier in der Registry einschreiben.
     *
     * @param handler
     *            Handler, der das Interface <i>IQrCodeHandler</i> implementiert.
     */
    public static void register(final IQrCodeHandler handler) {
        if (Logger.isLogging(Level.INFO)) {
            Logger.info("Registriere " + handler.getHandlerName() + " in QrCode Handler.");
        }
        QrCodeHandlerRegsistry.qrCodeHandlers.add(handler);
    }

    /**
     * Liefert einen passenden QrCodeHandler zum <b>qrCode</b>, falls vorhanden.
     *
     * @param qrCode
     *            <String> mit QR-Code zu dem ein Handler gesucht wird.
     * @return Handler von Typ <i>IQrCodeHandler</i>, der den String bearbeiten kann.
     */
    public static IQrCodeHandler getQrCodeHandler(final String qrCode) {
        IQrCodeHandler handler = null;
        final ListIterator<IQrCodeHandler> iter = QrCodeHandlerRegsistry.qrCodeHandlers.listIterator();
        while (iter.hasNext()) {
            handler = iter.next();
            if (handler.canHandle(qrCode)) {
                break;
            }
        }
        //
        return handler;
    }

    /**
     * Untersucht Eingabe-String, ob er mit einem Handler fuer Ueberweisung bearbeitet werden kann
     *
     * @param qrCode
     *            <String> mit QR-Code der untersucht werden soll.
     * @return <b>true</b> wenn QR-Code als berweisung bearbeitet werden kann, <b>false</> sonst.
     */
    public static boolean isUeberweisung(final String qrCode) {
        Boolean isUeberweisung = false;
        final ListIterator<IQrCodeHandler> iter = QrCodeHandlerRegsistry.qrCodeHandlers.listIterator();
        while (iter.hasNext()) {
            final IQrCodeHandler handler = iter.next();
            if (handler.canHandle(qrCode) && handler.isUeberweisung()) {
                isUeberweisung = true;
                break;
            }
        }
        return isUeberweisung;
    }
}
