/*******************************************************************************
 * Copyright 2017 Frank Mnster
 *
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.jameica.qrscanner;

import de.willuhn.jameica.system.Application;
import de.willuhn.util.I18N;

/**
 * Settings Klasse fr den jameica.qrscanner Basiert auf den Settings-Klasse aus
 * dem Beispiel-Plugin von Olaf Willuhn.
 *
 * @author Frank Mnster
 *
 * @see <a href=
 *      "http://www.willuhn.de/products/jameica/dev.php">http://www.willuhn.de/products/jameica/dev.php</a>
 */
public final class Settings {

	/**
	 * System settings von Jameica
	 */
	private static de.willuhn.jameica.system.Settings jameicaSettings = Application.getPluginLoader()
			.getPlugin(QrCodeScanner.class).getResources().getSettings();

	/**
	 * I18N local Variable
	 */
	private static I18N i18n;

	/**
	 * Is Hibiscus plugin also installed
	 */
	private static Boolean hibiscusInstalled;

	/**
	 * Standardkonstruktor
	 */
	private Settings() {
		// Absichtlich leer
	}

	/**
	 * Small helper function to get the translator.
	 *
	 * @return translator.
	 */
	public static I18N getI18n() {
		if (Settings.i18n == null) {
			Settings.i18n = Application.getPluginLoader().getPlugin(QrCodeScanner.class).getResources().getI18N();
		}
		return Settings.i18n;
	}

	/**
	 * berprft, ob das Hibiscus -Plugin installiert ist.
	 *
	 * @return isHibiscusInstalled <b>true</b>, wenn Hibiscus installiert ist,
	 *         <b>false</b> sonst
	 */
	public static Boolean isHibiscusInstalled() {
		if (Settings.hibiscusInstalled == null) {
			Settings.hibiscusInstalled = Application.getPluginLoader().isInstalled("de.willuhn.jameica.hbci.HBCI");
		}
		return Settings.hibiscusInstalled;
	}

	/**
	 * Soll IpStreaming Webcam verwendet werden.
	 *
	 * @param useIpStreamDriver
	 */
	public static void setUseIpStreamDriver(final Boolean useIpStreamDriver) {
		Settings.jameicaSettings.setAttribute("qrcode.useIpStreamDriver", useIpStreamDriver);
	}

	/**
	 * Liefert zur&uuml;ck, ob die IpStreaming Webcam verwendet werden soll.
	 *
	 * @return verwende IpStreamining Webcam
	 */
	public static boolean isUseIpStreamDriver() {
		return Settings.jameicaSettings.getBoolean("qrcode.useIpStreamDriver", false);
	}

	/**
	 * Liefert zur&uuml;ck, ob DroidCam App als Webcam verwendet werden soll.
	 *
	 * @return verwende DroidCam App als Webcam
	 */
	public static boolean isUseDroidCam() {
		return Settings.jameicaSettings.getBoolean("qrcode.useDroidCam", false);
	}

	/**
	 * Setzt, ob DroidCam App als Webcam verwendet werden soll.
	 *
	 * @param useDroidCam
	 *            benutzte DroidCam.
	 */
	public static void setUseDroidCam(final Boolean useDroidCam) {
		Settings.jameicaSettings.setAttribute("qrcode.useDroidCam", useDroidCam);
	}

	/**
	 * Liefert den DroidCam Host.
	 *
	 * @return den DroidCam Host.
	 */
	public static String getDroidCamHost() {
		return Settings.jameicaSettings.getString("qrcode.droidCamHost", "");
	}

	/**
	 * Setzt den DroidCam Host.
	 *
	 * @return den DroidCam Host.
	 */
	public static void setDroidCamHost(final String droidCamHost) {
		Settings.jameicaSettings.setAttribute("qrcode.droidCamHost", droidCamHost);
	}

	/**
	 * Liefert den gespeicherten DroidCam Port
	 *
	 * @return den DroidCam Port
	 */
	public static Integer getDroidCamPort() {
		return Settings.jameicaSettings.getInt("qrcode.droidCamPort", 4747);
	}

	/**
	 * Setzt den DroidCam Port.
	 *
	 * @return den DroidCam port.
	 */
	public static void setDroidCamPort(final int droidCamPort) {
		Settings.jameicaSettings.setAttribute("qrcode.droidCamPort", droidCamPort);
	}
}
