/*******************************************************************************
 * Copyright 2017 Frank Mnster
 *
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.jameica.qrscanner.gui.controller;

import java.net.MalformedURLException;
import java.util.List;

import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;

import com.github.sarxos.webcam.Webcam;
import com.github.sarxos.webcam.WebcamDevice;
import com.github.sarxos.webcam.ds.ipcam.IpCamDeviceRegistry;

import de.frankmuenster.jameica.qrscanner.Settings;
import de.frankmuenster.jameica.qrscanner.WebcamWatchdog;
import de.frankmuenster.jameica.qrscanner.gui.views.SettingsView;
import de.frankmuenster.jameica.qrscanner.ipcam.DroidCamDevice;
import de.frankmuenster.jameica.qrscanner.utils.Validators;
import de.willuhn.jameica.gui.AbstractControl;
import de.willuhn.jameica.gui.Action;
import de.willuhn.jameica.gui.GUI;
import de.willuhn.jameica.gui.input.CheckboxInput;
import de.willuhn.jameica.gui.input.IntegerInput;
import de.willuhn.jameica.gui.input.TextAreaInput;
import de.willuhn.jameica.gui.input.TextInput;
import de.willuhn.jameica.gui.parts.Button;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * Kontrolleur fr die Settings View
 *
 * @author munstfr
 *
 */
public class SettingsController extends AbstractControl {

	/**
	 * Die View in der wird die Einstellungen malen
	 */
	// private SettingsView view;

	/**
	 * Checkbox Eingabefeld ob DroidCam benutzt wird
	 */
	private transient CheckboxInput useDroidCam;

	/**
	 * DroidCam Host
	 */
	private transient TextInput droidCamHost;

	/**
	 * DroidCam Port
	 */
	private transient IntegerInput droidCamPort;

	/**
	 * Anzeigebreich fr gefundene Webcams
	 */
	private transient TextAreaInput discoverdWebcams;

	/**
	 * Aktualisiern Button
	 */
	private transient Button refreshBtn;

	/**
	 * Erstellt eine neue Instanz des Kontrollers
	 *
	 * @param view
	 * @throws MalformedURLException
	 */
	public SettingsController(final SettingsView view) {
		super(view);
	}

	/**
	 *
	 * @return Ein CheckBoxInput zur Auswahl, ob die DroidCam App als Wecam
	 *         benutzt werden soll.
	 */
	public CheckboxInput getUseDroidCam() {
		if (this.useDroidCam == null) {

			Logger.debug("UseDroidCam: " + Settings.isUseDroidCam());
			this.useDroidCam = new CheckboxInput(Settings.isUseDroidCam());
			this.useDroidCam.addListener(new Listener() {

				/**
				 * @see Listener#handleEvent(Event)
				 */
				@Override
				public void handleEvent(final Event event) {
					SettingsController.this.handleUseDroidCamEvent(event);
				}
			});
		}

		return this.useDroidCam;
	}

	/**
	 * Liefert ein Texteingabefeld f&uuml;r den DroidCam Host
	 *
	 * @return TextInput f&uuml;r DroidCam IP
	 */
	public TextInput getDroidCamHost() {
		if (this.droidCamHost == null) {
			this.droidCamHost = new TextInput(Settings.getDroidCamHost(), 15, "192.178.0.200"); // NOPMD
		}
		return this.droidCamHost;
	}

	/**
	 * Liefert ein DecimalInput f&uuml;r den DroidCam Port.
	 *
	 * @return TextInput f&uuml;r DroidCam Port
	 */
	public IntegerInput getDroidCamPort() {
		if (this.droidCamPort == null) {
			this.droidCamPort = new IntegerInput(Settings.getDroidCamPort());
			this.droidCamPort.setComment("");
		}
		return this.droidCamPort;
	}

	/**
	 * Liefert ein Anzeigebereich fr gefundene Webcams.
	 *
	 * @return TextAreaInput zum Anzeigen gefundener Webcams.
	 */
	public TextAreaInput getDiscoverdWebcams() {

		if (this.discoverdWebcams == null) {
			this.discoverdWebcams = new TextAreaInput(Settings.getI18n().tr("Nothing found"));
			this.discoverdWebcams.setHeight(130);
			this.discoverdWebcams.setComment("");
		}
		final List<Webcam> webcams = Webcam.getWebcams();

		final StringBuffer webcamNames = new StringBuffer();
		for (final Webcam webcam : webcams) {
			webcamNames.append(webcam.getName()).append('\n');
		}
		this.discoverdWebcams.setValue(webcamNames.toString());

		return this.discoverdWebcams;

	}

	/**
	 * Liefert einen Aktualisierungs-Button
	 *
	 * @return einen Aktualisierungs-Button
	 */
	public Button getRefeshBtn() {
		if (this.refreshBtn == null) {
			this.refreshBtn = new Button(Settings.getI18n().tr("Refresh"), new Action() {

				/**
				 * Action#handleAction(Object)
				 */
				@Override
				public void handleAction(final Object context) throws ApplicationException {
					Webcam.resetDriver();
					Webcam.setDriver(WebcamWatchdog.getDriver());
					Webcam.getDefault();

					final StringBuffer webcamNames = new StringBuffer();
					final List<WebcamDevice> dev = Webcam.getDriver().getDevices();
					for (final WebcamDevice webcamDevice : dev) {
						Logger.debug("Dev: " + webcamDevice.getName());
						webcamNames.append(webcamDevice.getName()).append('\n');
					}
					SettingsController.this.discoverdWebcams.setValue(webcamNames.toString());
					SettingsController.this.discoverdWebcams.getControl().redraw();
				}
			});
		}
		return this.refreshBtn;

	}

	/**
	 * Speichert die aktuellen Einstellungen im Settings Objekt.
	 */
	public void speichern() {

		// Check DroidCam IP if enabled
		final String droidCamIp = (String) this.droidCamHost.getValue();
		if ((boolean) this.useDroidCam.getValue() && !Validators.isValidInet4Address(droidCamIp)) {
			GUI.getStatusBar().setErrorText("DroidCam IP Adresse ist nicht korret formatiert.");

		} else {
			Settings.setUseDroidCam((Boolean) this.useDroidCam.getValue());
			Settings.setDroidCamHost((String) this.droidCamHost.getValue());
			Settings.setDroidCamPort((int) this.droidCamPort.getValue());
			GUI.getStatusBar().setSuccessText("Einstellungen gespeichert.");

			this.updateCamList();
		}

	}

	private void updateCamList() {

		if (IpCamDeviceRegistry.isRegistered("DroidCam")) {
			// ipCam.dispose();
			Logger.debug("Unregister : " + IpCamDeviceRegistry.unregister("DroidCam"));

		}
		if ((Boolean) this.useDroidCam.getValue()) {
			final DroidCamDevice ipCam = new DroidCamDevice((String) this.droidCamHost.getValue(),
					(int) this.droidCamPort.getValue());
			if (ipCam != null) {
				IpCamDeviceRegistry.register(ipCam);
			}
		}

		final StringBuffer webcamNames = new StringBuffer();
		final List<WebcamDevice> dev = Webcam.getDriver().getDevices();
		for (final WebcamDevice webcamDevice : dev) {
			Logger.debug("Dev: " + webcamDevice.getName());
			// Logger.debug("Dev: " + webcamDevice.;
			webcamNames.append(webcamDevice.getName()).append('\n');
		}
		this.discoverdWebcams.setValue(webcamNames.toString());
		this.discoverdWebcams.getControl().update();
	}

	/**
	 * @param event
	 */
	protected void handleUseDroidCamEvent(final Event event) {
		// TODO Automatisch generierter Methodenstub
		Logger.debug("useDroidCam Event" + event.toString());

		if ((boolean) this.useDroidCam.getValue()) {
			Logger.debug("Enable droidCam Fields");
			this.droidCamHost.enable();
			this.droidCamPort.enable();
		} else {
			Logger.debug("Disable droidCam Fields");
			this.droidCamHost.disable();
			this.droidCamPort.disable();
		}

	}

}
