/*******************************************************************************
 * Copyright 2017 Frank Mnster
 *
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/
package de.frankmuenster.jameica.qrscanner.gui.dialog;

import javax.swing.SwingUtilities;

import org.apache.commons.lang.NullArgumentException;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;

import com.github.sarxos.webcam.Webcam;

import de.frankmuenster.jameica.qrscanner.Settings;
import de.frankmuenster.jameica.qrscanner.gui.dialog.threads.QrCodeFoundListener;
import de.frankmuenster.jameica.qrscanner.gui.dialog.threads.QrCodeScannerThread;
import de.frankmuenster.jameica.qrscanner.gui.dialog.threads.WebcamSelectedListener;
import de.frankmuenster.jameica.qrscanner.gui.parts.FileImageDropPastePart;
import de.frankmuenster.jameica.qrscanner.gui.parts.ImageLabelPart;
import de.frankmuenster.jameica.qrscanner.gui.parts.WebcamSelector;
import de.willuhn.jameica.gui.Action;
import de.willuhn.jameica.gui.dialogs.AbstractDialog;
import de.willuhn.jameica.gui.input.TextAreaInput;
import de.willuhn.jameica.gui.parts.Button;
import de.willuhn.jameica.gui.parts.ButtonArea;
import de.willuhn.jameica.gui.util.Container;
import de.willuhn.jameica.gui.util.SimpleContainer;
import de.willuhn.jameica.system.OperationCanceledException;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;
import de.willuhn.util.I18N;

/**
 * Klasse die einen modalen Dialog erstellt und einen Thread startet, in dem Bilder der default Webcam angezeigt und
 * nach QRCodes gescannt werden. <br/>
 * In Version 1.1 ist das Droppen/ffnen von PDF Dateien hinzugekommen. In Version 1.3 die Mglichkeit eine Bild per
 * Zwischenablage zu scannen. Der Dialog schliet sich automatisch, wenn ein QR Code gefunden wurde. <br/>
 * Das Scannen kann mit Abbrechen unterbrochen werden
 *
 * @author Frank Mnster
 */
public class QrScannerDialog extends AbstractDialog<String> implements QrCodeFoundListener, WebcamSelectedListener {

    /**
     *
     */
    private static final String STARTING_SCANTASK_FAILED = "Could not start scanning task."; // NOPMD

    /**
     * i18n bersetzer aus unseren Settings
     */
    private final static I18N I18N = Settings.getI18n();

    /**
     * Enthlt das Scan-Ergebnis, also den QR-Code als String, oder eben nix
     */
    private transient String result;

    /**
     * Ein Webcam Selector Panel
     */
    private transient WebcamSelector webcamSelector;

    /**
     * Ein Label, um Webcam Infos anzuzeigen
     */
    private transient TextAreaInput webcamInfo;

    /**
     * Das ImageLabelPart Part, in dem das Webcam Bild dargestellt werden soll
     */
    private transient ImageLabelPart imgLabel;

    /**
     * der Hintergrund Thread indem die Webcam stndig nach neuen Bilder abgefragt und diese nach QR-Codes gescannt
     * werden
     */
    private transient QrCodeScannerThread scanTask;

    /**
     * Das Part zum droppen/ffnen von PDF Dateien
     */
    // private transient FileImageDropPastePart fileImgDropLbl;
    /**
     * Standard Konstruktor<br/>
     * Setzt Position, Gr&ouml;&szlig;e und Titel
     */
    public QrScannerDialog() {
        super(AbstractDialog.POSITION_CENTER, false);
        this.setTitle(QrScannerDialog.I18N.tr("Scanning qr-code"));
        this.setSize(330, 550);
    }

    /**
     * @see de.willuhn.jameica.gui.dialogs.AbstractDialog#paint(Composite)
     */
    @Override
    protected void paint(final Composite parent) throws Exception {
        final GridLayout layout = new GridLayout();
        layout.numColumns = 1;
        // layout.makeColumnsEqualWidth = true;
        parent.setLayout(layout);
        // Erstelle einfachen Container
        final Container container = new SimpleContainer(parent, true, 1);
        // Webcam-Auswahlbox
        this.webcamSelector = new WebcamSelector();
        this.webcamSelector.addListener(this);
        container.addPart(this.webcamSelector);
        container.addSeparator();
        final Webcam webcam = Webcam.getWebcamByName(this.webcamSelector.getValue());
        this.webcamInfo = new TextAreaInput("\n\n\n");
        this.webcamInfo.setHeight(60);
        container.addPart(this.webcamInfo);
        container.addSeparator();
        String text = "";
        // Ist berhaupt ein Webcam angeschlossen und verfgbar
        if (webcam == null) {
            text = "Keine Webcam verbunden oder die Webcam wird " + "leider nicht untersttzt!\n "
                    + "Die Funktionen zum Einesen von PDF-Dateien " + "stehen natrlich zur Verfgung.";
            this.setSize(330, 250);
        } else {
            // Erstelle schnes ImageLable in anstndiger Gre
            final Point size = new Point(320, 240);
            this.imgLabel = new ImageLabelPart(size);
            container.addPart(this.imgLabel);
            container.addSeparator();
        }
        this.webcamInfo.setValue(text);
        // drop target
        final FileImageDropPastePart fileImgDropLbl = new FileImageDropPastePart();
        fileImgDropLbl.addListener(this);
        container.addPart(fileImgDropLbl);
        // Abbrechen Button einfgen
        final Button cancle = new Button(QrScannerDialog.I18N.tr("Cancel"), new Action() {

            /**
             * @see de.willuhn.jameica.gui.Action#handleAction(Object)
             */
            @Override
            public void handleAction(final Object context) throws ApplicationException {
                QrScannerDialog.this.cancleTask();
                throw new OperationCanceledException();
            }
        }, null, false, "process-stop.png");
        final ButtonArea buttons = new ButtonArea();
        buttons.addButton(cancle);
        container.addButtonArea(buttons);
        // String webcamName = webcamSelector.getValue();
        if (webcam != null) {
            // Scan-Task starten
            try {
                this.doScanning();
            } catch (final NullArgumentException e) {
                final StringBuilder builder = new StringBuilder();
                builder.append(QrScannerDialog.I18N.tr(QrScannerDialog.STARTING_SCANTASK_FAILED)).append(' ')
                        .append(QrScannerDialog.I18N.tr("Please close the dialog and try again."));
                this.imgLabel.setText(builder.toString());
                Logger.error(QrScannerDialog.I18N.tr(QrScannerDialog.STARTING_SCANTASK_FAILED));
            }
        }
    }

    /**
     * @see de.willuhn.jameica.gui.dialogs.AbstractDialog#getData()
     */
    @Override
    protected String getData() throws Exception {
        /*
         * berprfe, ob der scanTask noch luft. Das kann passieren, wenn der Benutzer den Dialog mittels des X
         * schliet.
         */
        if (this.scanTask != null && !this.scanTask.isInterrupted()) {
            this.cancleTask();
        }
        return this.result;
    }

    /**
     * Scan-Task unterbrechen
     */
    private void cancleTask() {
        if (this.scanTask != null) {
            this.scanTask.removeListener(this);
            this.scanTask.interrupt();
            try {
                Thread.sleep(500);
            } catch (final InterruptedException e) {
            }
        }
    }

    /**
     * Generiert und implementiert den BackgroundTask zur Webcambildabfrage und QR-Code scannen.
     *
     * @throws Exception
     *             wenn Scanner-Task nicht erfolgreich gestartet werden konnte.
     */
    private void doScanning() throws NullArgumentException {
        final String webcamName = this.webcamSelector.getValue();
        this.scanTask = new QrCodeScannerThread(this.imgLabel, this.webcamInfo, webcamName);
        this.scanTask.addListener(this);
        SwingUtilities.invokeLater(this.scanTask);
    }

    /**
     * Listener for QR-Code gefunden. Im Fehlerfall is das <b>result</b> gleich <b>null</b>
     */
    @Override
    public void notifyOfQrCodeFound(final String result) {
        // Ergebnis sichern
        this.result = result;
        // wichtig Dialog schlieen
        this.close();
    }

    /**
     * @see de.frankmuenster.jameica.qrscanner.gui.dialog.threads.WebcamSelectedListener#notifyOfWebcamsChanged()
     */
    @Override
    public void notifyOfWebcamsChanged() {
        final String webcamName = this.webcamSelector.getValue();
        Logger.debug("Neue Webcam ausgewhlt: " + webcamName);
        this.cancleTask();
        while (this.scanTask.isAlive()) {
            try {
                Logger.debug("ScanTask still alive");
                Thread.sleep(500);
            } catch (final InterruptedException e) {
                Logger.trace("Noch eine Interrptanfrage ?!");
            }
        }
        try {
            this.doScanning();
        } catch (final NullArgumentException e) {
            final StringBuilder builder = new StringBuilder();
            builder.append(QrScannerDialog.I18N.tr(QrScannerDialog.STARTING_SCANTASK_FAILED)).append(' ')
                    .append(QrScannerDialog.I18N.tr("Please close the dialog and try again."));
            this.imgLabel.setText(builder.toString());
            Logger.error(QrScannerDialog.I18N.tr(QrScannerDialog.STARTING_SCANTASK_FAILED));
        }
    }
}
