/*******************************************************************************
 * Copyright 2017 Frank Mnster
 *
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/
package de.frankmuenster.jameica.qrscanner.gui.parts; // NOPMD

import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.rmi.RemoteException;
import java.util.Arrays;
import java.util.EnumMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.CopyOnWriteArraySet;

import javax.imageio.ImageIO;

import org.apache.commons.lang.NullArgumentException;
import org.apache.pdfbox.pdmodel.PDDocument;
import org.apache.pdfbox.rendering.ImageType;
import org.apache.pdfbox.rendering.PDFRenderer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.dnd.Clipboard;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DropTarget;
import org.eclipse.swt.dnd.DropTargetAdapter;
import org.eclipse.swt.dnd.DropTargetEvent;
import org.eclipse.swt.dnd.FileTransfer;
import org.eclipse.swt.dnd.ImageTransfer;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowData;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Label;

import com.google.zxing.BinaryBitmap;
import com.google.zxing.DecodeHintType;
import com.google.zxing.LuminanceSource;
import com.google.zxing.NotFoundException;
import com.google.zxing.Result;
import com.google.zxing.common.HybridBinarizer;
import com.google.zxing.multi.qrcode.QRCodeMultiReader;

import de.frankmuenster.jameica.qrcodehandler.QrCodeHandlerRegsistry;
import de.frankmuenster.jameica.qrscanner.Settings;
import de.frankmuenster.jameica.qrscanner.gui.dialog.threads.QrCodeFoundListener;
import de.frankmuenster.jameica.qrscanner.gui.utils.GUIUtil;
import de.frankmuenster.jameica.qrscanner.utils.BufferedImageLuminanceSource;
import de.frankmuenster.jameica.qrscanner.utils.GraphicsUtils;
import de.willuhn.jameica.gui.GUI;
import de.willuhn.jameica.gui.Part;
import de.willuhn.logging.Level;
import de.willuhn.logging.Logger;
import de.willuhn.util.I18N;

/**
 * Ein nettes kleines de.willuhn.jameica.gui.Part welches ein Drop Label, einen
 * ffnen Button und ein Status Label enthlt.<br/>
 * <br/>
 * PDF Dateien mit QR-Code berweisungen knnen per Drag&Drop auf das Drop-Label
 * gezogen werden oder mit dem ffnen-Button per File Dialogbox ausgewhlt
 * werden. Mit Paste knnen Screenshots aus dem Zwischenspeicher eingelesen
 * werden.<br/>
 * Im Statusfeld werden Statusinformationen, wie die aktuell bearbeitet Seite
 * oder ein negatives Suchergebnis angezeigt.
 *
 * @author Frank Mnster
 */
@SuppressWarnings({ "PMD.ModifiedCyclomaticComplexity", "PMD.CyclomaticComplexity" })
public class FileImageDropPastePart implements Part {

	/**
	 * Maximal Lnge eines Dateinamens in der Stausanzeige
	 */
	private static final int STATUSTEXT_MAX_FILENAME_LENGTH = 23; // NOPMD

	/**
	 * i18n bersetzer aus unseren Settings
	 */
	private final static I18N I18N = Settings.getI18n();

	/**
	 * Untersttzte Dateiformate
	 */
	private static final List<String> SUPPORTED_FILE_TYPES = Arrays.asList(".pdf", ".png", ".jpg", ".jpeg", ".bmp", // NOPMD
			".gif");

	/**
	 * Unsere Listeners fr QrCodeFound
	 */
	private transient final Set<QrCodeFoundListener> listeners = new CopyOnWriteArraySet<>();

	/**
	 * Composite fr das gesamte DropLabel
	 */
	private transient Composite parent;
	/**
	 * Der eigentliche Drop-Button
	 */

	// private transient Button buttonDrop;
	/**
	 * Das Status-label
	 */
	private transient Label lblStatus;
	/**
	 * Der Status-Text
	 */
	// private transient String status;

	/**
	 * Fgt eine Listener hinzu
	 *
	 * @param listener
	 *            QrCodeFoundListener der hinzugefgt werden soll
	 */
	public final void addListener(final QrCodeFoundListener listener) {
		this.listeners.add(listener);
	}

	/**
	 * Entfernd einen Listener aus der Liste
	 *
	 * @param listener
	 *            QrCodeFoundListener der entfernt werden soll
	 */
	public final void removeListener(final QrCodeFoundListener listener) {
		this.listeners.remove(listener);
	}

	/**
	 * Benachrichtigen der registrierten Listeners mit bergabe des gefundenen
	 * Strings
	 *
	 * @param result
	 *            (String) der gefundene QR-Code als String
	 */
	private void notifyListeners(final String result) {
		for (final QrCodeFoundListener listener : this.listeners) {
			listener.notifyOfQrCodeFound(result);
		}
	}

	/**
	 * Erstellt das ImageLable ohne weitere Inhalte
	 */
	public FileImageDropPastePart() {
		// Absichtlich leer
	}

	/**
	 * @see de.willuhn.jameica.gui.Part#paint(org.eclipse.swt.widgets.Composite)
	 */
	@Override
	@SuppressWarnings({ "PMD.NPathComplexity", "PMD.ExcessiveMethodLength" })
	public void paint(final Composite parent) throws RemoteException {
		// final GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL
		// | GridData.FILL_VERTICAL);
		if (parent == null) {
			Logger.error("Parent is null");
		}
		final GridData gdata = (GridData) parent.getLayoutData();
		// gd.horizontalSpan = 1;
		gdata.grabExcessHorizontalSpace = true;
		gdata.grabExcessVerticalSpace = false;
		parent.setLayoutData(gdata);
		// neues Composite erstellen, dass Text und Buttons auf nimmt
		this.parent = new Composite(parent, SWT.NONE);
		final GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 1;
		this.parent.setLayout(gridLayout);
		// neues Composite erstellen, welches Platz fuer den Kommentar laesst.
		final Composite lableRow = new Composite(this.parent, SWT.NONE);
		final RowLayout rowLayout = new RowLayout();
		rowLayout.wrap = false;
		rowLayout.pack = true;
		rowLayout.justify = true;
		rowLayout.type = SWT.HORIZONTAL;
		rowLayout.marginLeft = 5;
		rowLayout.marginTop = 5;
		rowLayout.marginRight = 5;
		rowLayout.marginBottom = 5;
		rowLayout.spacing = 0;
		rowLayout.center = true;
		lableRow.setLayout(rowLayout);
		// DropLabel erstellen
		final Button buttonDrop = new Button(lableRow, SWT.FLAT);
		// Image image = SWTUtil.getImage("drop_pdf.png");
		// buttonDrop.setImage(image);
		buttonDrop.setLayoutData(new RowData(180, 39));
		buttonDrop.setText("Ziehe Datei hierher...");
		// drop target
		final FileTransfer fTransfer = FileTransfer.getInstance();
		// final ImageTransfer iTransfer = ImageTransfer.getInstance();
		final Transfer[] transfers = new Transfer[] { fTransfer };
		final int ops = DND.DROP_COPY | DND.DROP_LINK | DND.DROP_DEFAULT;
		final DropTarget target = new DropTarget(buttonDrop, ops);
		target.setTransfer(transfers);
		target.addDropListener(new DropTargetAdapter() {

			/**
			 * @see org.eclipse.swt.dnd.DropTargetAdapter#drop
			 */
			@Override
			public void drop(final DropTargetEvent event) {
				if (event.data instanceof String[]) {
					final String[] filenames = (String[]) event.data;
					if (filenames.length > 0) {
						// bring Jameica to front
						GUIUtil.bringToFront(FileImageDropPastePart.this.getParent().getShell());
						final String file = filenames[0];
						Logger.debug("Filename: " + file);
						final String fileType = file.substring(file.lastIndexOf('.')).toLowerCase(Locale.getDefault());
						if (FileImageDropPastePart.SUPPORTED_FILE_TYPES.contains(fileType)) {
							FileImageDropPastePart.this.doImport(file);
						} else {
							FileImageDropPastePart.this
									.setStatusText(FileImageDropPastePart.I18N.tr("File type is not supported"));
							Logger.info("Datei ist keine PDF-Datei");
						}
					}
				}
			}

			/**
			 * @see org.eclipse.swt.dnd.DropTargetAdapter#dragEnter
			 */
			@Override
			public void dragEnter(final DropTargetEvent event) {
				event.detail = DND.DROP_COPY;
			}
		});
		// FileDialog oeffnen Button
		final Button button = new Button(lableRow, SWT.PUSH);
		button.setText("...");
		button.setLayoutData(new RowData(40, 39));
		button.addSelectionListener(new SelectionAdapter() {

			/**
			 * @see org.eclipse.swt.events.SelectionAdapter#widgetSelected
			 */
			@Override
			public void widgetSelected(final SelectionEvent event) {
				final FileDialog dialog = new FileDialog(GUI.getShell(), SWT.OPEN);
				final String[] filertExt = { "*.pdf;*.png;*.jpg;*.jpeg;*.bmp;*.gif" };
				dialog.setFilterExtensions(filertExt);
				final String file = dialog.open();
				Logger.debug("Selected file: " + file);
				if (file != null) {
					FileImageDropPastePart.this.doImport(file);
				}
			}
		});
		// Seperator
		final Label seperatorLabel = new Label(lableRow, SWT.SEPARATOR | SWT.VERTICAL);
		final RowData seperatoLayout = new RowData();
		seperatoLayout.height = 30;
		seperatoLayout.width = 20;
		seperatorLabel.setLayoutData(seperatoLayout);
		// Paste Button
		final Clipboard clipboard = new Clipboard(GUI.getDisplay());
		final Button pasteButton = new Button(lableRow, SWT.PUSH);
		pasteButton.setText("Paste");
		pasteButton.setLayoutData(new RowData(60, 39));
		pasteButton.addSelectionListener(new SelectionAdapter() {

			/**
			 * @see org.eclipse.swt.events.SelectionAdapter#widgetSelected
			 */
			@Override
			public void widgetSelected(final SelectionEvent event) {
				final List<String> typesList = Arrays.asList(clipboard.getAvailableTypeNames());
				if (typesList.contains("FileName")) {
					final String[] fileData = (String[]) clipboard.getContents(FileTransfer.getInstance());
					if (fileData != null) {
						boolean isSupported = false;
						// Es wurden Dateien kopiert
						for (final String file : fileData) {
							final String fileType = file.substring(file.lastIndexOf('.'))
									.toLowerCase(Locale.getDefault());
							if (FileImageDropPastePart.SUPPORTED_FILE_TYPES.contains(fileType)) {
								isSupported = true;
								FileImageDropPastePart.this.doImport(file);
							}
						}
						if (!isSupported) {
							FileImageDropPastePart.this
									.setStatusText(FileImageDropPastePart.I18N.tr("No supported file type found"));
						}
					}
				} else if (typesList.contains("CF_BITMAP")) {
					final ImageData imageData = (ImageData) clipboard.getContents(ImageTransfer.getInstance());
					if (imageData != null) {
						FileImageDropPastePart.this.doPaste(imageData);
					}
				} else {
					FileImageDropPastePart.this
							.setStatusText(FileImageDropPastePart.I18N.tr("Neither a file nor an screenshot found"));
				}
			}
		});
		// Status Feld
		this.lblStatus = new Label(this.parent, SWT.NONE);
	}

	/**
	 * Setzt den Statustext innerhalb des DropLabelParts
	 *
	 * @param status
	 *            der Statustext der angezeigt werden soll
	 */
	protected void setStatusText(final String status) {
		// this.status = status;
		if (this.lblStatus != null) {
			// final String t = status;
			Display.getDefault().asyncExec(new Runnable() {

				/**
				 * @see java.lang.Runnable#run()
				 */
				@Override
				public void run() {
					// ... do any work that updates the screen ...
					if (!FileImageDropPastePart.this.lblStatus.isDisposed()) {
						FileImageDropPastePart.this.lblStatus.setText(status);
						FileImageDropPastePart.this.lblStatus.redraw();
						FileImageDropPastePart.this.lblStatus.update();
						FileImageDropPastePart.this.lblStatus.getParent().layout();
					}
				}
			});
		}
	}

	/**
	 * Liefert das Composite, in dem das Control gemalt werden soll.
	 *
	 * @return das Composite, in dem das Control platziert wird.
	 */
	protected Composite getParent() {
		return this.parent;
	}

	/**
	 * Ldt eine Datei und untersucht sie auf enthaltene QR-Codes. Es
	 *
	 * @param file
	 *            Pfad und Dateiname der zu untersuchenden Datei als String.
	 */
	private void doImport(final String file) {
		final String inFile = file;
		Logger.debug("Importing file: " + inFile);
		final Thread thread = new Thread() {

			/**
			 * @see java.lang.Thread#run()
			 */
			@Override
			public void run() {
				try {
					String result = null;
					if (inFile.toLowerCase(Locale.getDefault()).endsWith(".pdf")) {
						result = FileImageDropPastePart.this.qrDecodePDF(inFile);
					} else {
						// Bild einlesen
						final BufferedImage image = ImageIO.read(new File(inFile));
						result = FileImageDropPastePart.this.qrDecode(image);
					}
					if (result == null) {
						final StringBuilder status = new StringBuilder(50);
						status.append(FileImageDropPastePart.I18N.tr("No qr-code found in")).append(' ');
						if (inFile.length() > FileImageDropPastePart.STATUSTEXT_MAX_FILENAME_LENGTH) {
							status.append("...").append(inFile.substring(inFile.length() - 20)); //$NON-NLS-1$
						} else {
							status.append(inFile);
						}
						FileImageDropPastePart.this.setStatusText(status.toString());
					} else {
						Logger.debug("QR-Code decoded successfully!"); //$NON-NLS-1$
						FileImageDropPastePart.this.notifyListeners(result);
					}
				} catch (final IOException e) {
					if (Logger.isLogging(Level.INFO)) {
						Logger.info("Datei " + inFile + " konnte nicht decodiert werden.");
					}
				}
			}
		};
		thread.start();
	}

	/**
	 * Untersucht ein Screenshot (oder andere ImageData-Objekt auf enthaltene
	 * QR-Codes
	 *
	 * @param image
	 *            zu untersuchendes SWT Image.
	 */
	private void doPaste(final ImageData image) {
		final BufferedImage img = GraphicsUtils.convertToAWT(image);
		Logger.debug("Untersuche Bild...");
		final Thread thread = new Thread() {

			/**
			 * @see java.lang.Thread#run()
			 */
			@Override
			public void run() {
				final String result = FileImageDropPastePart.this.qrDecode(img);
				if (result == null) {
					final String status = FileImageDropPastePart.I18N.tr("No qr-code found in pasted screenshot.");
					FileImageDropPastePart.this.setStatusText(status);
				} else {
					Logger.debug("QR-Code decoded successfully!");
					FileImageDropPastePart.this.notifyListeners(result);
				}
			}
		};
		thread.start();
	}

	/**
	 * Untersucht ein PDF Datei Seitenweise auf enthaltene QR-Codes. Dabei
	 * werden auch mehrere QR-Codes auf einer Seite gefunden und analysiert. Die
	 * Suche wird ggf. bei der ersten gefundenen berweisung in QR-Code-Format
	 * abgebrochen.
	 *
	 * @param pdfFile
	 *            die zu untersuchende PDF-Datei
	 * @return (String) aus dem gefundenen QR-Code, <b>null</b> sonst.
	 * @throws NullArgumentException
	 *             wenn die angegebene PDF-Datei null ist
	 * @throws IOException
	 *             wenn die angegeben PDF-Datei nicht geffnet werden kann
	 */
	private String qrDecodePDF(final String pdfFile) throws NullArgumentException, IOException {
		Logger.debug("In qrdebugPDF");
		if (pdfFile == null) {
			throw new NullArgumentException("File name cannot be null.");
		}
		ImageIO.scanForPlugins();
		final PDDocument document = PDDocument.load(new File(pdfFile));
		final PDFRenderer pdfRenderer = new PDFRenderer(document);
		String finalResult = null;
		for (int pageCounter = 0; pageCounter < document.getPages().getCount(); pageCounter++) {
			final String status = FileImageDropPastePart.I18N.tr("Analyse page") + " " + (pageCounter + 1) + " "
					+ FileImageDropPastePart.I18N.tr("of") + " " + document.getPages().getCount();
			Logger.debug(status);
			this.setStatusText(status);
			// note that the page number parameter is zero based
			final BufferedImage bim = pdfRenderer.renderImageWithDPI(pageCounter, 600, ImageType.GRAY);
			finalResult = this.qrDecode(bim);
			if (finalResult != null) {
				break;
			}
		}
		document.close();
		return finalResult;
	}

	/**
	 * Untersucht ein Image nach untersttzten QR-Codes und bricht beim ersten
	 * gefunden ab.
	 *
	 * @param img
	 *            BufferedImage das untersucht wird.
	 * @return (String) aus dem gefundenen QR-Code, <b>null</b> sonst.
	 */
	private String qrDecode(final BufferedImage img) {
		String result = null;
		final LuminanceSource source = new BufferedImageLuminanceSource(img);
		final BinaryBitmap bitmap = new BinaryBitmap(new HybridBinarizer(source));
		// new QRCodeReader();
		final QRCodeMultiReader qrMultiReader = new QRCodeMultiReader();
		// if (!bitmap.isRotateSupported()) {
		// }
		Map<DecodeHintType, Object> tmpHintsMap;
		tmpHintsMap = new EnumMap<>(DecodeHintType.class);
		tmpHintsMap.put(DecodeHintType.TRY_HARDER, Boolean.TRUE);
		try {
			final Result[] results = qrMultiReader.decodeMultiple(bitmap, tmpHintsMap);
			for (final Result r : results) {
				if (QrCodeHandlerRegsistry.isUeberweisung(r.getText())) {
					result = r.getText();
					break; // eine berweisung wurde gefunden, hier brechen
							// wir ab
				}
			}
		} catch (final NotFoundException e) {
			Logger.info("Kein QR-Code gefunden.");
		}
		return result;
	}
}
