/*******************************************************************************
 * Copyright 2017 Frank Mnster
 *  
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 * 
 * http://ec.europa.eu/idabc/eupl
 * 
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.jameica.qrscanner.gui.views;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.TabFolder;

import de.frankmuenster.jameica.qrscanner.Settings;
import de.frankmuenster.jameica.qrscanner.gui.controller.SettingsController;
import de.willuhn.jameica.gui.AbstractView;
import de.willuhn.jameica.gui.Action;
import de.willuhn.jameica.gui.GUI;
import de.willuhn.jameica.gui.parts.ButtonArea;
import de.willuhn.jameica.gui.util.ColumnLayout;
import de.willuhn.jameica.gui.util.SimpleContainer;
import de.willuhn.jameica.gui.util.TabGroup;
import de.willuhn.util.ApplicationException;
import de.willuhn.util.I18N;

/**
 * View fr die Einstellungen
 *
 * @author munstfr
 * @version 1.0
 */
public class SettingsView extends AbstractView {

    /**
     *
     */
    private static final String PLACEHOLDER_TEXT = "     ";

    /**
     * Unser bersetzer
     */
    private final static I18N I18N = Settings.getI18n();

    /**
     * Unser Kontrolleur
     */
    private final transient SettingsController control;

    /**
     * Der Tabfolder fr die einzelnen Tabs
     */
    private transient TabFolder folder;

    /**
     * Liefert eine neu Settings-View zurck und erstellt einen passenden Controller
     */
    public SettingsView() {
        super();
        this.control = new SettingsController(this);
    }

    /**
     * @see de.willuhn.jameica.gui.AbstractView#bind()
     */
    @Override
    public void bind() throws Exception {
        GUI.getView().setTitle(SettingsView.I18N.tr("Settings")); //$NON-NLS-1$
        final GridLayout grid = new GridLayout(2, false);
        grid.verticalSpacing = 10;
        final TabGroup grundEinstellung = new TabGroup(this.getTabFolder(), SettingsView.I18N.tr("Basic Settings")); //$NON-NLS-1$
        final ColumnLayout colLayout = new ColumnLayout(grundEinstellung.getComposite(), 2);
        final SimpleContainer left = new SimpleContainer(colLayout.getComposite());
        final SimpleContainer right = new SimpleContainer(colLayout.getComposite());
        left.addHeadline(SettingsView.I18N.tr("IP Streaming Support")); //$NON-NLS-1$
        left.addLabelPair(SettingsView.I18N.tr("Use DroidCam App"), this.control.getUseDroidCam()); //$NON-NLS-1$
        final ColumnLayout cl1 = new ColumnLayout(left.getComposite(), 2);
        final SimpleContainer host = new SimpleContainer(cl1.getComposite());
        host.addLabelPair("DroidCam Host and Port", this.control.getDroidCamHost());
        final SimpleContainer port = new SimpleContainer(cl1.getComposite());
        port.addInput(this.control.getDroidCamPort());
        left.addText(SettingsView.PLACEHOLDER_TEXT, false);
        left.addHeadline(SettingsView.I18N.tr("Webcams found"));
        left.addPart(this.control.getDiscoverdWebcams());
        left.addText(SettingsView.PLACEHOLDER_TEXT, false);
        left.addPart(this.control.getRefeshBtn());
        right.addText(SettingsView.PLACEHOLDER_TEXT, false);
        final ButtonArea buttons = new ButtonArea();
        buttons.addButton(SettingsView.I18N.tr("Save"), new Action() {

            /**
             * @see de.willuhn.jameica.gui.Action#handleAction(java.lang.Object)
             */
            @Override
            public void handleAction(final Object context) throws ApplicationException {
                SettingsView.this.control.speichern();
            }
        }, null, true, "document-save.png");
        buttons.paint(this.getParent());
    }

    /**
     * Liefert den Tab-Folder, in dem die einzelnen Module der Einstellungen untergebracht sind.
     *
     * @return der Tab-Folder.
     */
    private TabFolder getTabFolder() {
        if (this.folder == null) {
            this.folder = new TabFolder(this.getParent(), SWT.NONE);
            this.folder.setLayoutData(new GridData(GridData.FILL_BOTH));
        }
        return this.folder;
    }
}
