/*******************************************************************************
 * Copyright 2017 Frank Mnster
 *  
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 * 
 * http://ec.europa.eu/idabc/eupl
 * 
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.jameica.qrscanner.utils;

import java.util.HashSet;
import java.util.Set;

/**
 * ContryCode Klasse zum validiern, ob IBAN udn BIC zu EUCountry gehoeren. Aus Forumeintrag
 * http://www.tutorials.de/java/259520-iban-bic.html
 *
 * @see <a href="www.tutorials.de/java/259520-iban-bic.html">http://www.tutorials.de/java/259520-iban-bic.html</a>
 * @author takidoso / munstfr
 * @version 1.0
 */
public final class EUCountryCodes {

    /**
     * Liste der Lnder fr einen gltiger IBAN Code
     */
    static private Set<String> ibanCountryCodes = new HashSet<>();

    /**
     * Liste der Lnder fr einen gltiger BIC Code
     */
    static private Set<String> bicCountryCodes = new HashSet<>();
    static {
        EUCountryCodes.ibanCountryCodes.add("AT");
        EUCountryCodes.ibanCountryCodes.add("BE");
        EUCountryCodes.ibanCountryCodes.add("CY");
        EUCountryCodes.ibanCountryCodes.add("CZ");
        EUCountryCodes.ibanCountryCodes.add("DE");
        EUCountryCodes.ibanCountryCodes.add("DK");
        EUCountryCodes.ibanCountryCodes.add("EE");
        EUCountryCodes.ibanCountryCodes.add("ES");
        EUCountryCodes.ibanCountryCodes.add("FI");
        EUCountryCodes.ibanCountryCodes.add("FR");
        EUCountryCodes.ibanCountryCodes.add("GB");
        EUCountryCodes.ibanCountryCodes.add("GI");
        EUCountryCodes.ibanCountryCodes.add("GR");
        EUCountryCodes.ibanCountryCodes.add("HU");
        EUCountryCodes.ibanCountryCodes.add("IE");
        EUCountryCodes.ibanCountryCodes.add("IS");
        EUCountryCodes.ibanCountryCodes.add("IT");
        EUCountryCodes.ibanCountryCodes.add("LI");
        EUCountryCodes.ibanCountryCodes.add("LT");
        EUCountryCodes.ibanCountryCodes.add("LU");
        EUCountryCodes.ibanCountryCodes.add("LV");
        EUCountryCodes.ibanCountryCodes.add("MT");
        EUCountryCodes.ibanCountryCodes.add("NL");
        EUCountryCodes.ibanCountryCodes.add("NO");
        EUCountryCodes.ibanCountryCodes.add("PL");
        EUCountryCodes.ibanCountryCodes.add("PT");
        EUCountryCodes.ibanCountryCodes.add("SE");
        EUCountryCodes.ibanCountryCodes.add("SI");
        EUCountryCodes.ibanCountryCodes.add("SK");
        EUCountryCodes.bicCountryCodes.add("AT");
        EUCountryCodes.bicCountryCodes.add("BE");
        EUCountryCodes.bicCountryCodes.add("CY");
        EUCountryCodes.bicCountryCodes.add("CZ");
        EUCountryCodes.bicCountryCodes.add("DE");
        EUCountryCodes.bicCountryCodes.add("DK");
        EUCountryCodes.bicCountryCodes.add("EE");
        EUCountryCodes.bicCountryCodes.add("ES");
        EUCountryCodes.bicCountryCodes.add("FI");
        EUCountryCodes.bicCountryCodes.add("FR");
        EUCountryCodes.bicCountryCodes.add("GB");
        EUCountryCodes.bicCountryCodes.add("IE");
        EUCountryCodes.bicCountryCodes.add("GF");
        EUCountryCodes.bicCountryCodes.add("GI");
        EUCountryCodes.bicCountryCodes.add("GP");
        EUCountryCodes.bicCountryCodes.add("GR");
        EUCountryCodes.bicCountryCodes.add("HU");
        EUCountryCodes.bicCountryCodes.add("IE");
        EUCountryCodes.bicCountryCodes.add("IS");
        EUCountryCodes.bicCountryCodes.add("IT");
        EUCountryCodes.bicCountryCodes.add("LI");
        EUCountryCodes.bicCountryCodes.add("LT");
        EUCountryCodes.bicCountryCodes.add("LU");
        EUCountryCodes.bicCountryCodes.add("LV");
        EUCountryCodes.bicCountryCodes.add("MQ");
        EUCountryCodes.bicCountryCodes.add("MT");
        EUCountryCodes.bicCountryCodes.add("NL");
        EUCountryCodes.bicCountryCodes.add("NO");
        EUCountryCodes.bicCountryCodes.add("PL");
        EUCountryCodes.bicCountryCodes.add("PT");
        EUCountryCodes.bicCountryCodes.add("RE");
        EUCountryCodes.bicCountryCodes.add("SE");
        EUCountryCodes.bicCountryCodes.add("SI");
        EUCountryCodes.bicCountryCodes.add("SK");
    }

    private EUCountryCodes() {
    }

    /**
     * Ueberprueft Eingabestring (im BIC-Format) auf Laenderkennung an Stelle 5 und 6.
     * 
     * @param bic
     *            <Sting> zu ueberpruefende BIC
     * @return <b>true</b>, wenn EU Laenderkennung gefunden, <b>false</b> sonst.
     */
    static public boolean isBicEUCountry(final String bic) {
        return EUCountryCodes.bicCountryCodes.contains(bic.substring(4, 6));
    }

    /**
     * Ueberprueft Eingabestring (im IBAN-Format) auf Laenderkennung an Stelle 1 und 2.
     * 
     * @param iban
     *            <Sting> zu berprfende IBAN
     * @return <b>true</b>, wenn EU Lnderkennung gefunden, <b>false</b> sonst.
     */
    static public boolean isIbanEUCountry(final String iban) {
        return EUCountryCodes.ibanCountryCodes.contains(iban.substring(0, 2));
    }
}
